/* FsHandler.cpp */
#include "mbed.h"
#include "FsHandler.h"
//#define DEBUG
#include "hl_debug.h"
#include "stdlib.h"

DigitalOut led_red(LED1);
DigitalOut led_green(LED2);
DigitalOut led_blue(LED3);

void SNTP_Connect(void);
void time_set(void);
void time_update(char buffer[]);

extern int hour_r, min_r,hm_r, hm_n;
extern int flag_t;
extern double duty;

PwmOut myservo(P10);
datetime ntptime;
struct tm timeinfo;

uint8_t update_mode = 0;
uint8_t Set_update[3] = {18, 00, 00};   //Hour,Minute,Second

//ymd_buffer[0]~[3] : Year
//ymd_buffer[4]~[5] : Month
//ymd_buffer[6]~[7] : Day
extern char ymd_buffer[8];
//hms_buffer[0]~[1] : Hour
//hms_buffer[2]~[3] : Minute
//hms_buffer[4]~[5] : Second
extern char hms_buffer[6];

static int matchstrings(const char* one, const char* two)
{
    int m = 0;
    
    for (m = 0; m < min(strlen(one), strlen(two)) ; m++) {
        if (one[m] != two[m])
            return m;
    }
    return m;
}

std::map<const char*, const char*> HTTPFsRequestHandler::m_fsMap;
 
HTTPFsRequestHandler::HTTPFsRequestHandler(const char* rootPath, const char* localPath, HTTPConnection::HTTPMessage& Msg, TCPSocketConnection& Tcp)
    : HTTPRequestHandler(Msg, Tcp)
{
    m_rootPath = rootPath;
    m_localPath = localPath;
    
    string myPath = m_rootPath + m_localPath;
    
    //  Now replace the virtual root path with a mounted device path
    std::map<const char*, const char*>::iterator it;
    const char *bestMatch = NULL;
    const char *bestMatchExchange = NULL;
    int match_ind = -1;
    for (it = m_fsMap.begin() ; it != m_fsMap.end() ; it++) {
        //  find best match (if the given logical path is containted in the root
        int s = matchstrings(myPath.c_str(), it->second);
        INFO("Matching Root %s with handler %s results in %d identical characters\n", myPath.c_str(), it->second, s);
        if ((s == strlen(it->second)) && (s > match_ind)) {
            match_ind = s;
            bestMatch = it->first;
            bestMatchExchange = it->second;
        }
    }

    if (bestMatch != NULL) {
        m_rootPath = bestMatch;
        m_localPath = string(myPath).substr(strlen(bestMatchExchange));
    }
            
    handleRequest();
}

HTTPFsRequestHandler::~HTTPFsRequestHandler()
{
}

std::map<int, EthernetInterface*> HTTPFsRequestHandler::m_eth_list;

int HTTPFsRequestHandler::handleGetRequest()
{
    HTTPHeaders headers;
    EthernetInterface test_eth;
    int retval = 0;   //success
    
    if( std::string::npos != msg.uri.find("get_netinfo.cgi") )
    {}/*
        char buf[256];
                
        sprintf(buf, "NetinfoCallback({\"mac\":\"%s\",\"ip\":\"%s\",\"sn\":\"%s\",\"gw\":\"%s\",\"temp\":\"%s\"});"
                    ,m_eth_list[0]->getMACAddress(),m_eth_list[0]->getIPAddress(),m_eth_list[0]->getNetworkMask(),m_eth_list[0]->getGateway());
                
        tcp.send(buf, strlen(buf));
    }
    
    else if( std::string::npos != msg.uri.find("get_ain.cgi") )
    {        
        char buf[256];
        
        //sprintf(buf, "AinCallback({\"ain_v0\":\"%.3f\",\"ain_v1\":\"%.3f\"});",ain0.read()*100, ain1.read()*100);
                
        tcp.send(buf, strlen(buf));
    }*/
    
    else //read html pages
    {
        if (m_localPath.length() > 4) 
            getStandardHeaders(headers, m_localPath.substr(m_localPath.length()-4).c_str());
        else
            getStandardHeaders(headers);
        
        INFO("Handling Get Request (root = %s, local = %s).", m_rootPath.c_str(), m_localPath.c_str());
        
        std::string reqPath;
    
        //  Check if we received a directory with the local bath
        if ((m_localPath.length() == 0) || (m_localPath.substr( m_localPath.length()-1, 1) == "/")) {
            //  yes, we shall append the default page name
            m_localPath += "index.html";
        }
        
        reqPath = m_rootPath + m_localPath;
        
        INFO("Mapping \"%s\" to \"%s\"", msg.uri.c_str(), reqPath.c_str());
            
        FILE *fp = fopen(reqPath.c_str(), "r");
        if (fp != NULL) {
            char * pBuffer = NULL;
            int sz = 8192;
            while( pBuffer == NULL) {
                sz /= 2;
                pBuffer = (char*)malloc(sz);
                if (sz < 128)
                    error ("OutOfMemory");
            }
            
            //  File was found and can be returned
        
            //  first determine the size
            fseek(fp, 0, SEEK_END);
            long size = ftell(fp);
            fseek(fp, 0, SEEK_SET);
        
            startResponse(200, size);
            while(!feof(fp) && !ferror(fp)) {
                int cnt = fread(pBuffer, 1, sz , fp);
                if (cnt < 0)
                    cnt = 0;
                processResponse(cnt, pBuffer);
            }
            delete pBuffer;
            endResponse();
            fclose(fp);
        }
        else {
            retval = 404;
            ERR("Requested file was not found !");
        }
    }
    
    return retval;
}

int HTTPFsRequestHandler::handlePostRequest()
{
    char ch;    
    myservo.period_ms(20);
    int pin = 0;
    
    if( std::string::npos != msg.uri.find("set_dio.cgi") )
    {
        pin = get_http_param_value("pin");
        if(pin == 8)
        {
            led_red = get_http_param_value("val");
        }
        else if(pin == 9)
        {
            led_green = get_http_param_value("val");
        }
        else if(pin == 5)
        {
            led_blue = get_http_param_value("val");
        }
        else
        {
            WARN("Wrong pin number");
        }   
        
        return 0;
    }
    else if( std::string::npos != msg.uri.find("set_dio2.cgi") )
    {
        ch = get_http_param_value("pin");
        printf("ch : %d \r\n",ch);
                              
        if(ch == 0)
        {
            duty = 0.05;
            myservo = duty;
            printf("duty : %lf\r\n", duty);
        }
        else if(ch == 1)
        {
            duty = 1;
            myservo = duty;
            printf("duty : %lf\r\n", duty);
            //myservo.pulsewidth_ms(1.0); 
        }
        else if(ch == 2)
        {
            duty = 0.2;
            myservo = duty;
            printf("duty : %lf\r\n", duty);
            //myservo.pulsewidth_ms(1.0); 
        }      
        else if(ch == 3)
        {
            duty = 0.2;
            myservo = duty;
            printf("duty : %lf\r\n", duty);
            wait(5);
            duty = 1;
            myservo = duty;
            printf("duty : %lf\r\n", duty);
        } 
    }
    else if( std::string::npos != msg.uri.find("set_time.cgi") )
    {
        hour_r = get_http_param_value("hour");
        min_r = get_http_param_value("min");
        
        printf("hour : %d, min : %d\r\n", hour_r, min_r);
        
        SNTP_Connect();
        time_set();
        
        time_t seconds = time(NULL);
        
        
        strftime(ymd_buffer, 8, "%Y%m%d%\n\r", localtime(&seconds)); 
        
        if(hms_buffer[4] == '0' && hms_buffer[5] == '0')
        {
            time_update(hms_buffer);
            printf("update HMS : %s\r\n",hms_buffer);
        }
        
        hm_r = (hour_r*100) + min_r;
        hm_n = atoi(hms_buffer);
        hm_n = hm_n / 100;
        printf("HMr : %d\r\n",hm_r);
        printf("HMn : %d\r\n",hm_n);
        
        flag_t = 1;
    }
    else
    {
           return 404;
    }
}

int HTTPFsRequestHandler::handlePutRequest()
{
    return 404;
}

uint32_t HTTPFsRequestHandler::get_http_param_value(char* param_name)
{
    uint8_t * name = 0;
    uint8_t * pos2;
    uint16_t len = 0;
    char value[10];
    uint32_t ret = 0;
    
    //msg.attri
    if((name = (uint8_t *)strstr(msg.attri, param_name)))
    {
        name += strlen(param_name) + 1;
        pos2 = (uint8_t*)strstr((char*)name, "&");
        if(!pos2)
        {
            pos2 = name + strlen((char*)name);
        }
        len = pos2 - name;
        
        if(len)
        {
            strncpy(value, (char*)name, len);
            ret = atoi(value);
        }
    }
    return ret;
}

void SNTP_Connect(void) {
    printf("Getting time information by using NTP...\r\n");
    
    SNTPClient sntp("time.nist.gov", 40);   // timezone: Korea, Republic of
    sntp.connect();
    if(sntp.getTime(&ntptime) == true) {
          printf("%d-%d-%d, %02d:%02d:%02d\r\n", ntptime.yy, ntptime.mo, ntptime.dd, ntptime.hh, ntptime.mm, ntptime.ss);
          printf("Completed Get and Set Time\r\n\r\n");
    }
    else {
        while(sntp.getTime(&ntptime) == true) {
            printf("%d-%d-%d, %02d:%02d:%02d\r\n", ntptime.yy, ntptime.mo, ntptime.dd, ntptime.hh, ntptime.mm, ntptime.ss);
            printf("Completed Get and Set Time\r\n\r\n");
            break;
        }
    }
    sntp.close();
}

void time_set(void) {
    timeinfo.tm_mon = ntptime.mo-1;
    timeinfo.tm_mday = ntptime.dd;
    timeinfo.tm_hour = ntptime.hh;
    timeinfo.tm_min = ntptime.mm;
    timeinfo.tm_sec = ntptime.ss;
    timeinfo.tm_year = ntptime.yy-1900;
    time_t t =mktime(&timeinfo);
    set_time(t);
    t = time(NULL);
}

void time_update(char buffer[]) {
    uint8_t h_buffer = ((buffer[0]-48)*10) + (buffer[1]-48);
    uint8_t m_buffer = ((buffer[2]-48)*10) + (buffer[3]-48);
    uint8_t s_buffer = ((buffer[4]-48)*10) + (buffer[5]-48);

    if(update_mode == 0){
        if(h_buffer == Set_update[0] && m_buffer == Set_update[1] && s_buffer == Set_update[2]){
            SNTP_Connect();
            time_set();
            update_mode = 1;
            printf("Time Update Completed.\n\r\n\r");
        }
    }
    else if(update_mode == 1){
        if(!(h_buffer == Set_update[0] && m_buffer == Set_update[1] && s_buffer == Set_update[2])){
            update_mode = 0;
        }
    }
}
